import os
import unittest

from hovercraft.generate import rst2html
from hovercraft.template import Template
from .test_data import HTML_OUTPUTS

TEST_DATA = os.path.join(os.path.split(__file__)[0], "test_data")


class GeneratorTests(unittest.TestCase):
    """Tests that the resulting HTML is correct.

    This tests the whole path except the copying of files."""

    def test_small(self):
        template = Template(os.path.join(TEST_DATA, "minimal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "simple.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["simple"])

    def test_big(self):
        template = Template(os.path.join(TEST_DATA, "maximal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "advanced.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["advanced"])

    def test_presenter_notes(self):
        template = Template(os.path.join(TEST_DATA, "maximal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "presenter-notes.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["presenter-notes"])

    def test_skip_presenter_notes(self):
        template = Template(os.path.join(TEST_DATA, "maximal"))
        html, deps = rst2html(
            os.path.join(TEST_DATA, "presenter-notes.rst"), template, skip_notes=True
        )
        self.assertEqual(html, HTML_OUTPUTS["skip-presenter-notes"])

    def test_comments(self):
        template = Template(os.path.join(TEST_DATA, "minimal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "comment.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["comment"])

    def test_slide_with_class(self):
        template = Template(os.path.join(TEST_DATA, "minimal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "slide_class.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["slide_with_class"])

    def test_table(self):
        template = Template(os.path.join(TEST_DATA, "minimal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "table.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["table"])

    def test_class_directive(self):
        template = Template(os.path.join(TEST_DATA, "minimal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "class.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["class_directive"])

    def test_container_directive(self):
        template = Template(os.path.join(TEST_DATA, "minimal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "container.rst"), template)
        self.assertEqual(html, HTML_OUTPUTS["container_directive"])

    def test_include(self):
        template = Template(os.path.join(TEST_DATA, "minimal"))
        html, deps = rst2html(os.path.join(TEST_DATA, "include.rst"), template)
        self.assertIn(b"Presentation with an include</h1>", html)
        # Make sure the simple presentation was included:
        self.assertIn(b"Simple Presentation</h1>", html)


if __name__ == "__main__":
    unittest.main()



import os
import sys
from tempfile import TemporaryDirectory
import unittest

from hovercraft import main
from .test_data import HTML_OUTPUTS

TEST_DATA = os.path.join(os.path.split(__file__)[0], "test_data")


class HTMLTests(unittest.TestCase):
    """Test the procedure from rst to html"""

    def test_small(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                "bin/hovercraft",
                "-t" + os.path.join(TEST_DATA, "minimal"),
                os.path.join(TEST_DATA, "simple.rst"),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS["simple"])

            js_files = os.listdir(os.path.join(tmpdir, "js"))
            self.assertEqual(set(js_files), {"impress.js", "hovercraft-minimal.js"})

    def test_extra_css(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                "bin/hovercraft",
                "-t" + os.path.join(TEST_DATA, "maximal"),
                "-c" + os.path.join(TEST_DATA, "extra.css"),
                "-n",
                os.path.join(TEST_DATA, "simple.rst"),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS["extra_css"])

            out_files = os.listdir(tmpdir)
            self.assertEqual(
                set(out_files),
                {
                    "extra.css",
                    "index.html",
                    "js",
                    "css",
                    "images",
                    "fonts",
                    "directory",
                },
            )

    def test_extra_js(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                "bin/hovercraft",
                "-t" + os.path.join(TEST_DATA, "maximal"),
                "-j" + os.path.join(TEST_DATA, "extra.js"),
                "-n",
                os.path.join(TEST_DATA, "simple.rst"),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS["extra_js"])

            out_files = os.listdir(tmpdir)
            self.assertEqual(
                set(out_files),
                {"extra.js", "index.html", "js", "css", "images", "fonts", "directory"},
            )

    def test_big(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                "bin/hovercraft",
                "-t" + os.path.join(TEST_DATA, "maximal", "template.cfg"),
                os.path.join(TEST_DATA, "advanced.rst"),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                # We have verified the contents in test_generator.py, let's
                # just check that it writes the right thing:
                self.assertEqual(outfile.read(), HTML_OUTPUTS["advanced"])

            out_files = os.listdir(tmpdir)
            self.assertEqual(
                set(out_files),
                {
                    "extra.css",
                    "extra.js",
                    "index.html",
                    "js",
                    "css",
                    "images",
                    "fonts",
                    "directory",
                },
            )

            # Make sure the whole tree of the directory resource was copies,
            # except .dont-include, which should not be included.
            out_files = os.listdir(os.path.join(tmpdir, "directory"))
            self.assertEqual(
                set(out_files), {"subdir", "hovercraft_logo.png", "print.css"}
            )
            out_files = os.listdir(os.path.join(tmpdir, "directory", "subdir"))
            self.assertEqual(set(out_files), {"afile"})

            js_files = os.listdir(os.path.join(tmpdir, "js"))
            self.assertEqual(
                set(js_files),
                {"impress.js", "hovercraft.js", "impressConsole.js", "dummy.js"},
            )
            css_files = os.listdir(os.path.join(tmpdir, "css"))
            self.assertEqual(
                set(css_files), {"print.css", "style.css", "impressConsole.css"}
            )
            image_files = os.listdir(os.path.join(tmpdir, "images"))
            self.assertEqual(set(image_files), {"hovercraft_logo.png"})
            font_files = os.listdir(os.path.join(tmpdir, "fonts"))
            self.assertEqual(
                set(font_files),
                {
                    "texgyreschola-regular-webfont.ttf",
                    "texgyreschola-regular-webfont.eot",
                    "texgyreschola-regular-webfont.woff",
                    "texgyreschola-regular-webfont.svg",
                },
            )

    def test_skip_notes(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                "bin/hovercraft",
                "-t" + os.path.join(TEST_DATA, "maximal"),
                "-n",
                os.path.join(TEST_DATA, "presenter-notes.rst"),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                # We have verified the contents in test_generator.py, let's
                # just check that it writes the right thing:
                self.assertEqual(outfile.read(), HTML_OUTPUTS["skip-presenter-notes"])

            out_files = os.listdir(tmpdir)
            self.assertEqual(
                set(out_files),
                {"index.html", "js", "css", "images", "fonts", "directory"},
            )
            js_files = os.listdir(os.path.join(tmpdir, "js"))
            self.assertEqual(
                set(js_files),
                {"impress.js", "hovercraft.js", "impressConsole.js", "dummy.js"},
            )
            css_files = os.listdir(os.path.join(tmpdir, "css"))
            self.assertEqual(
                set(css_files), {"print.css", "style.css", "impressConsole.css"}
            )
            image_files = os.listdir(os.path.join(tmpdir, "images"))
            self.assertEqual(set(image_files), {"hovercraft_logo.png"})
            font_files = os.listdir(os.path.join(tmpdir, "fonts"))
            self.assertEqual(
                set(font_files),
                {
                    "texgyreschola-regular-webfont.ttf",
                    "texgyreschola-regular-webfont.eot",
                    "texgyreschola-regular-webfont.woff",
                    "texgyreschola-regular-webfont.svg",
                },
            )

    def test_default_template(self):
        with TemporaryDirectory() as tmpdir:
            # Adding a non-existant subdir, to test that it gets created.
            tmpdir = os.path.join(tmpdir, "foo")

            sys.argv = [
                "bin/hovercraft",
                os.path.join(TEST_DATA, "advanced.rst"),
                tmpdir,
            ]
            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS["default-template"])

            js_files = os.listdir(os.path.join(tmpdir, "js"))
            self.assertEqual(
                set(js_files), {"impress.js", "hovercraft.js", "gotoSlide.js"}
            )
            css_files = os.listdir(os.path.join(tmpdir, "css"))
            self.assertEqual(set(css_files), {"hovercraft.css", "highlight.css"})
            image_files = os.listdir(os.path.join(tmpdir, "images"))
            self.assertEqual(set(image_files), {"hovercraft_logo.png"})

    def test_auto_console(self):
        with TemporaryDirectory() as tmpdir:
            # Adding a non-existant subdir, to test that it gets created.
            tmpdir = os.path.join(tmpdir, "foo")

            sys.argv = [
                "bin/hovercraft",
                "-a",
                os.path.join(TEST_DATA, "simple.rst"),
                tmpdir,
            ]
            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                result = outfile.read()
                self.assertIn(b'auto-console="True"', result)

    def test_subdirectory_css(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                "bin/hovercraft",
                os.path.join(TEST_DATA, "subdir-css.rst"),
                tmpdir,
            ]

            main()

            out_files = os.listdir(tmpdir)
            self.assertEqual(set(out_files), {"index.html", "js", "css", "images"})
            css_files = os.listdir(os.path.join(tmpdir, "css"))
            self.assertEqual(
                set(css_files),
                {"hovercraft.css", "highlight.css", "sub.css", "sub2.css"},
            )
            image_files = os.listdir(os.path.join(tmpdir, "images"))
            self.assertEqual(set(image_files), {"hovercraft_logo.png"})

    def test_mathjax(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                "bin/hovercraft",
                os.path.join(TEST_DATA, "math.rst"),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, "index.html"), "rb") as outfile:
                result = outfile.read()
                self.assertIn(
                    b'<script type="text/javascript" '
                    b'src="https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.5/MathJax.js?config=TeX-MML-AM_CHTML">',
                    result,
                )
                self.assertIn(
                    br'<div class="math-block ">$$\begin{align}dS = \frac{dQ}{T}\end{align}$$</div>',
                    result,
                )
                self.assertIn(br'<span class="math ">\(S = k \log W\)</span>', result)


if __name__ == "__main__":
    unittest.main()



from docutils import nodes
from docutils.core import publish_string
from docutils.readers.standalone import Reader
from docutils.transforms.misc import Transitions
from docutils.writers.docutils_xml import Writer
from lxml import etree


class HovercraftTransitions(Transitions):
    @property
    def _doc_transitions(self):
        if not hasattr(self.document, "_transitions"):
            self.document._transitions = []
        return self.document._transitions

    def visit_transition(self, node):
        # First add the level of the transition as a class
        char = node.rawsource[0]
        if char not in self._doc_transitions:
            self._doc_transitions.append(char)
        level = self._doc_transitions.index(char) + 1
        node.attributes["level"] = str(level)

        index = node.parent.index(node)
        error = None
        # Here the default transformer complained if you had a transition as
        # the first thing in a section. I don't see why.
        if isinstance(node.parent[index - 1], nodes.transition):
            error = self.document.reporter.error(
                "At least one body element must separate transitions; "
                "adjacent transitions are not allowed.",
                source=node.source,
                line=node.line,
            )
        if error:
            # Insert before node and update index.
            node.parent.insert(index, error)
            index += 1
        assert index < len(node.parent)
        if index != len(node.parent) - 1:
            # No need to move the node.
            return
        # Node behind which the transition is to be moved.
        sibling = node
        # While sibling is the last node of its parent.
        while index == len(sibling.parent) - 1:
            sibling = sibling.parent
            # If sibling is the whole document (i.e. it has no parent).
            if sibling.parent is None:
                # Transition at the end of document.  Do not move the
                # transition up, and place an error behind.
                error = self.document.reporter.error(
                    "Document may not end with a transition.", line=node.line
                )
                node.parent.insert(node.parent.index(node) + 1, error)
                return
            index = sibling.parent.index(sibling)
        # Remove the original transition node.
        node.parent.remove(node)
        # Insert the transition after the sibling.
        sibling.parent.insert(index + 1, node)


class HovercraftReader(Reader):
    def get_transforms(self):
        transforms = Reader().get_transforms()
        transforms.remove(Transitions)
        transforms.append(HovercraftTransitions)
        return transforms


def rst2xml(rststring, source_path=None):
    reader = HovercraftReader()
    writer = Writer()
    result = publish_string(
        rststring,
        source_path=source_path,
        reader=reader,
        writer=writer,
        settings_overrides={"syntax_highlight": "short"},
    )
    dependencies = reader.settings.record_dependencies.list
    return result, dependencies


def copy_node(node):
    """Makes a copy of a node with the same attributes and text, but no children."""
    element = node.makeelement(node.tag)
    element.text = node.text
    element.tail = node.tail
    for key, value in node.items():
        element.set(key, value)

    return element


class SlideMaker(object):
    """A docutils XML walker that will organize the XML into slides"""

    def __init__(self, intree, skip_notes=False):
        self.intree = intree
        self.result = None
        self.curnode = None
        self.steps = 0
        self.skip_notes = skip_notes
        self.skip_nodes = (
            "docinfo",
            "field_list",
            "field",
            "field_body",
        )
        self.need_mathjax = False
        self.lists_with_substeps = []

    def _newstep(self, level):
        step = etree.Element(
            "step",
            attrib={"step": str(self.steps), "class": "step step-level-%s" % level,},
        )
        self.steps += 1
        return step

    def walk(self):
        walken = etree.iterwalk(self.intree, events=("start", "end"))
        for event, node in walken:
            if node.tag in self.skip_nodes:
                continue
            method = getattr(self, "%s_%s" % (event, node.tag), None)
            if method is None:
                if event == "start":
                    self.default_start(node)
                else:
                    if event == "end":
                        self.default_end(node)
            else:
                method(node)

        return self.result

    def default_start(self, node):
        new = copy_node(node)
        self.curnode.append(new)
        self.curnode = new

    def default_end(self, node):
        if self.curnode.tag != "step":
            self.curnode = self.curnode.getparent()

    def start_document(self, node):
        self.curnode = self.result = copy_node(node)

    def start_section(self, node):
        # If there has been no transition by now, start a new step:
        if self.steps == 0:
            step = self._newstep(1)
            self.result.append(step)
            self.curnode = step

        # Then carry on as normal
        self.default_start(node)

    def start_transition(self, node):
        level = int(node.attrib["level"])
        step = self._newstep(level)
        # Walk back up to a transition of the same level
        curnode = self.curnode

        def get_level(clss):
            for cls in clss.split():
                # Skip unrelated classes, such as those added by the user.
                if cls.startswith("step-level-"):
                    return int(cls.split("-")[-1])

        while curnode.tag != "step" or get_level(curnode.attrib["class"]) >= level:
            parent = curnode.getparent()
            if parent is None:
                # Top of tree
                break
            curnode = parent

        # This is the transition above the new transition.
        # Add the new transition to the end.
        curnode.append(step)
        self.curnode = step

    def end_transition(self, node):
        self.default_end(node)

    def start_field_name(self, node):
        # Fields are made into attribute, nothing to do here:.
        pass

    def end_field_name(self, node):
        # Fields are made into attributes
        pass

    def start_paragraph(self, node):
        # Fields are made into attributes.
        parent = node.getparent()
        if parent.tag == "field_body":
            fieldname = parent.getprevious().text
            current = self.curnode.get(fieldname)
            if current:
                value = current + " " + node.text
            else:
                value = node.text
            self.curnode.set(fieldname, value)
        else:
            self.default_start(node)

    def end_paragraph(self, node):
        parent = node.getparent()
        if parent.tag != "field_body":
            self.default_end(node)

    def start_note(self, node):
        if not self.skip_notes:
            return self.default_start(node)
        # Skip this node completely, including children:
        while len(node) > 0:
            del node[0]

    def start_math_block(self, node):
        self.need_mathjax = True
        self.default_start(node)

    def start_math(self, node):
        self.need_mathjax = True
        self.default_start(node)

    def start_list_item(self, node):
        if self.lists_with_substeps:
            # We are currently in a list hierarchy that should have
            # substeps
            classes = node.attrib.get("classes", "")
            if "substep" not in classes:
                classes += " substep"
                node.attrib["classes"] = classes.strip()

        self.default_start(node)

    def _fix_substep_list(self, node):
        # The list istelf should not have the substep class.
        # For some reason we can't modify nodes in the end_...() functions,
        # only in the start_...() functions, so we drop the substep class
        # but add the node to a stack of nodes with the class
        classes = node.attrib.get("classes", "")
        if "substep" in classes:
            self.lists_with_substeps.append(node)
            classes = classes.replace("substep", "").strip()
            if not classes:
                del node.attrib["classes"]
            else:
                node.attrib["classes"] = classes

    def start_enumerated_list(self, node):
        self._fix_substep_list(node)
        self.default_start(node)

    def end_enumerated_list(self, node):
        if node in self.lists_with_substeps:
            self.lists_with_substeps.remove(node)
        self.default_end(node)

    def start_bullet_list(self, node):
        self._fix_substep_list(node)
        self.default_start(node)

    def end_bullet_list(self, node):
        if node in self.lists_with_substeps:
            self.lists_with_substeps.remove(node)
        self.default_end(node)



import unittest
from pkg_resources import resource_string
from lxml import etree

from hovercraft.parse import SlideMaker, rst2xml


def make_tree(file_name):
    """Loads reStructuredText, outputs an lxml tree"""
    rst = resource_string(__name__, file_name)
    xml, dependencies = rst2xml(rst)
    return etree.fromstring(xml)


class SlideMakerTests(unittest.TestCase):
    """Test the conversion of docutils XML into XML suitable to give to the templates"""

    def test_simple(self):
        tree = SlideMaker(make_tree("test_data/simple.rst")).walk()
        self.assertEqual(
            etree.tostring(tree),
            (
                b'<document source="&lt;string&gt;"><step step="0" class="step step-level-1">'
                b'<section ids="simple-presentation" names="simple\\ presentation">'
                b"<title>Simple Presentation</title><paragraph>This presentation "
                b"has two slides, each with a header and some text.</paragraph>"
                b'</section></step><step step="1" class="step step-level-1"><section '
                b'ids="second-slide" names="second\\ slide"><title>Second slide</title><paragraph>'
                b"There is no positioning or anything fancy.</paragraph>"
                b"</section></step></document>"
            ),
        )

    def test_advanced(self):
        tree = SlideMaker(make_tree("test_data/advanced.rst")).walk()
        xml = etree.tostring(tree)
        target = (
            b'<document source="&lt;string&gt;" title="Presentation title" '
            b'data-transition-duration="2000" auto-console="True" '
            b'css-screen="extra.css" js-body="extra.js"><paragraph>This is an advanced '
            b"presentation. It doesn't have a section in the first\nstep, "
            b"meaning the first step will not be a step at all, but a sort of\n"
            b"introductory comment about the presentation, that will not show up "
            b"in the\npresentation at all.</paragraph><paragraph>It also sets a "
            b'title and a transition-duration.</paragraph><step step="0" class="step '
            b'step-level-1" data-x="1000" data-y="1600"><section '
            b'ids="advanced-presentation" names="advanced\\ presentation">'
            b"<title>Advanced Presentation</title><paragraph>Here we "
            b"show the positioning feature, where we can explicitly set a "
            b"position\non one of the steps.</paragraph></section></step><step "
            b'step="1" class="step step-level-1" id="name-this-step" '
            b'data-x="r1600"><section ids="formatting" names="formatting"><title>'
            b"Formatting</title><paragraph>Let us also try some basic "
            b"formatting, like <emphasis>italic</emphasis>, and <strong>bold"
            b'</strong>.</paragraph><bullet_list bullet="*"><list_item>'
            b"<paragraph>We can also</paragraph></list_item><list_item>"
            b"<paragraph>have a list</paragraph></list_item><list_item>"
            b"<paragraph>of things.</paragraph></list_item></bullet_list>"
            b'</section></step><step step="2" class="step step-level-1">'
            b"<paragraph>There should also be possible to have\n"
            b"preformatted text for code.</paragraph><literal_block "
            b'classes="code python" xml:space="preserve"><inline classes="k">'
            b'def</inline> <inline classes="nf">foo</inline><inline '
            b'classes="p">(</inline><inline classes="n">bar</inline><inline '
            b'classes="p">):</inline>\n    <inline classes="c1"># Comment'
            b'</inline>\n    <inline classes="n">a</inline> <inline '
            b'classes="o">=</inline> <inline classes="mi">1</inline> <inline '
            b'classes="o">+</inline> <inline classes="s2">"hubbub"</inline>'
            b'\n    <inline classes="k">return</inline> <inline classes="kc">'
            b'None</inline></literal_block></step><step step="3" class="step step-level-1">'
            b"<paragraph>An image, with attributes:</paragraph><image "
            b'classes="imageclass" uri="images/hovercraft_logo.png" width="50%"/>'
            b'</step><step step="4" class="step step-level-1"><section ids="character-sets" '
            b'names="character\\ sets"><title>Character sets</title><paragraph>'
            b"The character set is UTF-8 as of now. Like this: "
            b"&#229;&#228;&#246;.</paragraph></section></step></document>"
        )
        self.assertEqual(xml, target)

    def test_presenter_notes(self):
        tree = SlideMaker(make_tree("test_data/presenter-notes.rst")).walk()
        target = (
            b'<document ids="document-title" names="document\\ title" '
            b'source="&lt;string&gt;" title="Document title"><title>Document '
            b'title</title><step step="0" '
            b'class="step step-level-1"><section ids="hovercrafts-presenter-notes" '
            b'names="hovercrafts\\ presenter\\ notes"><title>Hovercrafts presenter '
            b"notes</title><paragraph>Hovercraft! supports presenter notes. It does "
            b'this by taking anything in a\nwhat is called a "notes-admonition" and '
            b"making that into presenter notes.</paragraph><note><paragraph>Hence, "
            b"this will show up as presenter notes.\nYou have still access to a lot "
            b'of formatting, like</paragraph><bullet_list bullet="*"><list_item>'
            b"<paragraph>Bullet lists</paragraph></list_item><list_item><paragraph>"
            b"And <emphasis>all</emphasis> types of <strong>inline formatting"
            b"</strong></paragraph></list_item></bullet_list></note></section>"
            b'</step><step step="1" class="step step-level-1"><image '
            b'uri="images/hovercraft_logo.png"/><note><paragraph>You '
            b"don't have to start the text on the same line as\nthe note, but "
            b"you can.</paragraph><paragraph>You can also have several paragraphs."
            b" You can not have any\nheadings of any kind "
            b"though.</paragraph><paragraph><strong>But you can fake them through "
            b"bold-text</strong></paragraph><paragraph>And that's useful enough "
            b"for presentation notes.</paragraph></note></step></document>"
        )
        self.assertEqual(etree.tostring(tree), target)

    def test_transition_levels(self):
        # Make the XML
        xml, deps = rst2xml(
            b"Intro\n\n====\n\nLevel 1\n\n====\n\nLevel 1\n\n----\n\nLevel 2\n\n"
            b"....\n\nLevel 3\n\n----\n\nLevel 2\n\n....\n\nLevel 3\n\n"
            b"====\n\nLevel 1"
        )

        target_start = (
            b'<?xml version="1.0" encoding="utf-8"?>\n<!DOCTYPE document PUBLIC "+'
            b'//IDN docutils.sourceforge.net//DTD Docutils Generic//EN//XML" '
            b'"http://docutils.sourceforge.net/docs/ref/docutils.dtd">\n'
        )
        target_end = (
            b'<document source="&lt;string&gt;"><paragraph>Intro</paragraph>'
            b'<transition level="1"></transition><paragraph>Level 1</paragraph>'
            b'<transition level="1"></transition><paragraph>Level 1</paragraph>'
            b'<transition level="2"></transition><paragraph>Level 2</paragraph>'
            b'<transition level="3"></transition><paragraph>Level 3</paragraph>'
            b'<transition level="2"></transition><paragraph>Level 2</paragraph>'
            b'<transition level="3"></transition><paragraph>Level 3</paragraph>'
            b'<transition level="1"></transition><paragraph>Level 1</paragraph>'
            b"</document>"
        )
        self.assertTrue(xml.startswith(target_start))
        self.assertTrue(xml.endswith(target_end))

        # Make the slides:
        tree = SlideMaker(etree.fromstring(xml)).walk()

        target = (
            b'<document source="&lt;string&gt;"><paragraph>Intro</paragraph>'
            b'<step step="0" class="step step-level-1"><paragraph>Level 1</paragraph></step>'
            b'<step step="1" class="step step-level-1"><paragraph>Level 1</paragraph>'
            b'<step step="2" class="step step-level-2"><paragraph>Level 2</paragraph>'
            b'<step step="3" class="step step-level-3"><paragraph>Level 3</paragraph></step>'
            b'</step><step step="4" class="step step-level-2"><paragraph>Level 2</paragraph>'
            b'<step step="5" class="step step-level-3"><paragraph>Level 3</paragraph></step>'
            b'</step></step><step step="6" class="step step-level-1"><paragraph>Level 1'
            b"</paragraph></step></document>"
        )
        self.assertEqual(etree.tostring(tree), target)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from lxml import etree

from hovercraft.template import (
    Template,
    CSS_RESOURCE,
    JS_RESOURCE,
    JS_POSITION_BODY,
    JS_POSITION_HEADER,
)

TEST_DATA = os.path.join(os.path.split(__file__)[0], "test_data")


class TemplateInfoTests(unittest.TestCase):
    """Tests that template information is correctly parsed"""

    def test_template_paths(self):
        # You can specify a folder or a cfg file and that's the same thing.
        template_info1 = Template(os.path.join(TEST_DATA, "minimal"))
        template_info2 = Template(os.path.join(TEST_DATA, "minimal", "template.cfg"))
        self.assertEqual(
            etree.tostring(template_info1.xml_node()),
            etree.tostring(template_info2.xml_node()),
        )

    def test_template_minimal(self):
        template_info = Template(os.path.join(TEST_DATA, "minimal"))
        with open(os.path.join(TEST_DATA, "minimal", "template.xsl"), "rb") as xslfile:
            xsl = xslfile.read()
        self.assertEqual(template_info.xsl, xsl)
        template_files = [each.filepath for each in template_info.resources]
        self.assertIn("js/impress.js", template_files)
        self.assertIn("js/hovercraft-minimal.js", template_files)
        css_files = list(
            each.filepath
            for each in template_info.resources
            if each.resource_type == CSS_RESOURCE
        )
        self.assertEqual(len(css_files), 0)
        self.assertEqual(template_info.doctype, b"<!DOCTYPE html>")

    def test_template_maximal(self):
        template_info = Template(os.path.join(TEST_DATA, "maximal"))
        with open(os.path.join(TEST_DATA, "maximal", "template.xsl"), "rb") as xslfile:
            xsl = xslfile.read()
        self.assertEqual(template_info.xsl, xsl)

        template_files = [each.filepath for each in template_info.resources]
        self.assertIn("images/hovercraft_logo.png", template_files)
        self.assertIn("js/impress.js", template_files)
        self.assertIn("js/impressConsole.js", template_files)
        self.assertIn("js/hovercraft.js", template_files)

        js_bodies = [
            each.filepath
            for each in template_info.resources
            if each.resource_type == JS_RESOURCE and each.extra_info == JS_POSITION_BODY
        ]
        self.assertIn("js/impress.js", js_bodies)
        self.assertIn("js/impressConsole.js", js_bodies)
        self.assertIn("js/hovercraft.js", js_bodies)

        js_headers = [
            each.filepath
            for each in template_info.resources
            if each.resource_type == JS_RESOURCE
            and each.extra_info == JS_POSITION_HEADER
        ]
        self.assertIn("js/dummy.js", js_headers)

        self.assertEqual(template_info.resources[0].filepath, "css/style.css")
        self.assertEqual(template_info.resources[0].extra_info, "all")
        self.assertEqual(template_info.resources[1].filepath, "css/print.css")
        self.assertEqual(template_info.resources[1].extra_info, "print")
        self.assertEqual(template_info.resources[2].filepath, "css/impressConsole.css")
        self.assertEqual(template_info.resources[2].extra_info, "screen,projection")

        self.assertEqual(
            template_info.doctype, b'<!DOCTYPE html SYSTEM "about:legacy-compat">'
        )


class TemplateInfoNodeTests(unittest.TestCase):
    """Tests that template information is correctly made into an xml nodes"""

    def test_minimal_template(self):
        template_info = Template(os.path.join(TEST_DATA, "minimal"))
        node = template_info.xml_node()

        self.assertEqual(
            etree.tostring(node),
            (
                b"<templateinfo><header/><body>"
                b'<js src="js/impress.js"/><js src="js/hovercraft-minimal.js"/>'
                b"</body></templateinfo>"
            ),
        )

    def test_maximal_template(self):
        template_info = Template(os.path.join(TEST_DATA, "maximal"))
        node = template_info.xml_node()

        self.assertEqual(
            etree.tostring(node),
            (
                b"<templateinfo><header>"
                b'<css href="css/style.css" media="all"/>'
                b'<css href="css/print.css" media="print"/>'
                b'<css href="css/impressConsole.css" media="screen,projection"/>'
                b'<js src="js/dummy.js"/></header>'
                b'<body><js src="js/impress.js"/><js src="js/impressConsole.js"/>'
                b'<js src="js/hovercraft.js"/>'
                b"</body></templateinfo>"
            ),
        )


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from pkg_resources import resource_string
from lxml import etree

from hovercraft.parse import rst2xml, SlideMaker
from hovercraft.position import gather_positions, calculate_positions, position_slides

TEST_DATA = os.path.join(os.path.split(__file__)[0], "test_data")


def make_tree(file_name):
    """Loads reStructuredText, outputs an lxml tree"""
    rst = resource_string(__name__, os.path.join("test_data", file_name))
    xml, deps = rst2xml(rst)
    return SlideMaker(etree.fromstring(xml)).walk()


class GatherTests(unittest.TestCase):
    """Tests that position information is correctly parsed"""

    def test_gathering(self):
        tree = make_tree("positioning.rst")

        positions = list(gather_positions(tree))

        self.assertEqual(
            positions,
            [
                {
                    "data-x": "r0",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "1",
                    "is_path": False,
                },
                {
                    "data-x": "r1600",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": False,
                },
                {
                    "data-x": "r1600",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": True,
                    "path": "m 100 100 l 200 0 l 0 200",
                },
                {
                    "data-x": "r1600",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": True,
                },
                {
                    "data-x": "r1600",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": True,
                },
                {
                    "data-x": "0",
                    "data-y": "0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": False,
                },
                {
                    "data-x": "r0",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "90",
                    "data-scale": "r0",
                    "is_path": False,
                },
                {
                    "data-x": "r0",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": False,
                },
                {
                    "data-x": "r0",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": True,
                    "path": "m 100 100 l 200 0 l 0 200",
                },
                {
                    "data-x": "r0",
                    "data-y": "r0",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": True,
                },
                {
                    "data-x": "r0",
                    "data-y": "r0",
                    "data-z": "1000",
                    "data-rotate-x": "180",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": True,
                },
                {
                    "data-x": "3000",
                    "data-y": "1000",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": False,
                },
                {
                    "data-x": "firstID+1000",
                    "data-y": "firstID-500",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": False,
                },
                {
                    "data-x": "secondID+800",
                    "data-y": "200",
                    "data-z": "r0",
                    "data-rotate-x": "r0",
                    "data-rotate-y": "r0",
                    "data-rotate-z": "r0",
                    "data-scale": "r0",
                    "is_path": False,
                },
            ],
        )


class CalculateTests(unittest.TestCase):
    """Tests that positions are correctly calculated"""

    def test_square(self):
        # Slides, positioned in a square
        positions = [
            {"data-x": "0", "data-y": "0"},
            {"data-x": "r1200", "data-y": "0"},
            {"data-x": "r1200", "data-y": "0"},
            {"data-x": "r1200", "data-y": "0"},
            {"data-x": "r0", "data-y": "r-1000"},
            {"data-x": "r0", "data-y": "r-1000"},
            {"data-x": "r0", "data-y": "r-1000"},
            {"data-x": "r-1200", "data-y": "r0"},
            {"data-x": "r-1200", "data-y": "r0"},
            {"data-x": "r-1200", "data-y": "r0"},
            {"data-x": "r0", "data-y": "r1000"},
            {"data-x": "r0", "data-y": "r1000"},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(
            positions,
            [
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1200,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 2400,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3600,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3600,
                    "data-y": -1000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3600,
                    "data-y": -2000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3600,
                    "data-y": -3000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 2400,
                    "data-y": -3000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1200,
                    "data-y": -3000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 0,
                    "data-y": -3000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 0,
                    "data-y": -2000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 0,
                    "data-y": -1000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
            ],
        )

    def test_relative_positioning(self):
        # Relative positioning is probably the most useful positioning.
        # It allows you to insert or remove a slide, and everything adjusts.
        positions = [
            # The first two slides are just default positons
            {"data-x": "r0", "data-y": "r0"},
            {"data-x": "r1600", "data-y": "r0"},
            # Then suddenly we move vertically!
            {"data-x": "r0", "data-y": "r1000"},
            # Continue the same way one slide.
            {"data-x": "r0", "data-y": "r1000"},
            # Stand still
            {"data-x": "r0", "data-y": "r0"},
            # Stand still again!
            {"data-x": "r0", "data-y": "r0"},
            # Move a little bit
            {"data-x": "r-40", "data-y": "r-200"},
            # Go back to normal movement to the right
            {"data-x": "r1600", "data-y": "r0"},
            {"data-x": "r1600", "data-y": "r0"},
            {"data-x": "r1600", "data-y": "r0"},
            # Absolute movement back to start!
            {"data-x": "0", "data-y": "0"},
            # Absolute movement to a center for end (with zoomout for example)
            {"data-x": "3000", "data-y": "1000"},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(
            positions,
            [
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1600,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1600,
                    "data-y": 1000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1600,
                    "data-y": 2000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1600,
                    "data-y": 2000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1600,
                    "data-y": 2000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1560,
                    "data-y": 1800,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3160,
                    "data-y": 1800,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 4760,
                    "data-y": 1800,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 6360,
                    "data-y": 1800,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3000,
                    "data-y": 1000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
            ],
        )

    def test_absolute_path(self):
        # Position slides along a path
        positions = [
            {
                "data-x": "r0",
                "data-y": "r0",
                "path": "M 100 100 L 300 100 L 300 300",
                "is_path": True,
            },
            {"is_path": True},
            {"is_path": True},
            {"is_path": True},
            {"is_path": True},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(
            positions,
            [
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 2000,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 4000,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 44.99999999999999,
                    "data-scale": 1,
                },
                {
                    "data-x": 4000,
                    "data-y": 2000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
                {
                    "data-x": 4000,
                    "data-y": 4000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
            ],
        )

    def test_relative_path(self):
        positions = [
            {"data-x": "r0", "data-y": "r0"},
            {"data-x": "r1600", "data-y": "r0"},
            {
                "data-x": "r1600",
                "data-y": "r0",
                "is_path": True,
                "path": "m 100 100 l 200 0 l 0 200",
            },
            {"data-x": "r0", "data-y": "r0", "is_path": True},
            {"data-x": "r0", "data-y": "r0", "is_path": True},
            {"data-x": "r1600", "data-y": "r0"},
            {"data-x": "r0", "data-y": "r2400"},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(
            positions,
            [
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1600,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3200,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                # This point is exactly on a 90 degree angle. Therefore,
                # it's angle is calculated as 45 degrees, it being the
                # average.
                {
                    "data-x": 5600,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 44.99999999999999,
                    "data-scale": 1,
                },
                {
                    "data-x": 5600,
                    "data-y": 2400,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
                {
                    "data-x": 7200,
                    "data-y": 2400,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
                {
                    "data-x": 7200,
                    "data-y": 4800,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
            ],
        )

    def test_complex_path(self):
        positions = [
            {"data-x": "r0", "data-y": "r0"},
            {"data-x": "r1600", "data-y": "r0"},
            {
                "data-x": "r1600",
                "data-y": "r0",
                "path": "m 100 100 l 200 0 l 0 200",
                "is_path": True,
            },
            {"is_path": True},
            {"is_path": True},
            # Note that we don't change the rotation, so it stays at 90, here.
            {"data-x": "0", "data-y": "0"},
            # No new x and y, previous was absolute: Stay still!
            {},
            {
                "data-x": "r0",
                "data-y": "r0",
                "path": "m 100 100 l 200 0 l 0 200",
                "is_path": True,
            },
            {"is_path": True},
            {"is_path": True},
            {"data-x": "3000", "data-y": "1000", "data-rotate-z": "0"},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(
            positions,
            [
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 1600,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3200,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 5600,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 44.99999999999999,
                    "data-scale": 1,
                },
                {
                    "data-x": 5600,
                    "data-y": 2400,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
                # Note that we don't change the rotation, so it stays at 90, here.
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
                # No settings, still same place and rotation.
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
                # We start a path, but x and y are r0, so no movement.
                # However, the rotation will come from the path, so it resets to 0.
                {
                    "data-x": 0,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
                {
                    "data-x": 2400,
                    "data-y": 0,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 44.99999999999999,
                    "data-scale": 1,
                },
                {
                    "data-x": 2400,
                    "data-y": 2400,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 90.0,
                    "data-scale": 1,
                },
                {
                    "data-x": 3000,
                    "data-y": 1000,
                    "data-z": 0,
                    "data-rotate-x": 0,
                    "data-rotate-y": 0,
                    "data-rotate-z": 0,
                    "data-scale": 1,
                },
            ],
        )


class PositionTest(unittest.TestCase):
    def test_complete(self):
        tree = make_tree("positioning.rst")
        # Position the slides:
        position_slides(tree)

        # Get all slide position data:

        positions = []
        for step in tree.findall("step"):
            pos = {}
            for key in step.attrib:
                if key.startswith("data-"):
                    pos[key] = step.attrib[key]

            positions.append(pos)

        self.assertEqual(
            positions,
            [
                {
                    "data-x": "0",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "0",
                    "data-scale": "1",
                },
                {
                    "data-x": "1600",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "0",
                    "data-scale": "1",
                },
                # Because of the path, we now get an explicit rotation:
                {
                    "data-x": "3200",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "0",
                    "data-scale": "1",
                },
                {
                    "data-x": "5600",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "44.99999999999999",
                    "data-scale": "1",
                },
                {
                    "data-x": "5600",
                    "data-y": "2400",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90.0",
                    "data-scale": "1",
                },
                # Rotation carries over from last part of path.
                {
                    "data-x": "0",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90.0",
                    "data-scale": "1",
                },
                # No position change
                {
                    "data-x": "0",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90",
                    "data-scale": "1",
                },
                # No change at all.
                {
                    "data-x": "0",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90",
                    "data-scale": "1",
                },
                # Path starts, rotation comes from path:
                {
                    "data-x": "0",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "0",
                    "data-scale": "1",
                },
                {
                    "data-x": "2400",
                    "data-y": "0",
                    "data-z": "0",
                    "data-rotate-x": "0",
                    "data-rotate-y": "0",
                    "data-rotate-z": "44.99999999999999",
                    "data-scale": "1",
                },
                # Explicit rotate-x and z, automatic position including rotate-z from path.
                {
                    "data-x": "2400",
                    "data-y": "2400",
                    "data-z": "1000",
                    "data-rotate-x": "180",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90.0",
                    "data-scale": "1",
                },
                # Explicit x and y, all other carry over from last slide.
                {
                    "data-x": "3000",
                    "data-y": "1000",
                    "data-z": "1000",
                    "data-rotate-x": "180",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90.0",
                    "data-scale": "1",
                },
                # Positioning relative to other slide by id
                {
                    "data-x": "4000",
                    "data-y": "500",
                    "data-z": "1000",
                    "data-rotate-x": "180",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90.0",
                    "data-scale": "1",
                },
                # Positioning x relative to other slide by id, Explicit y
                {
                    "data-x": "4800",
                    "data-y": "200",
                    "data-z": "1000",
                    "data-rotate-x": "180",
                    "data-rotate-y": "0",
                    "data-rotate-z": "90.0",
                    "data-scale": "1",
                },
            ],
        )


if __name__ == "__main__":
    unittest.main()



import os
import re
import shutil
from lxml import etree, html
from pkg_resources import resource_string

from .parse import rst2xml, SlideMaker
from .position import position_slides
from .template import (
    Template,
    CSS_RESOURCE,
    JS_RESOURCE,
    JS_POSITION_HEADER,
    JS_POSITION_BODY,
    OTHER_RESOURCE,
    DIRECTORY_RESOURCE,
)


class ResourceResolver(etree.Resolver):
    def resolve(self, url, pubid, context):
        if url.startswith("resource:"):
            prefix, filename = url.split(":", 1)
            return self.resolve_string(resource_string(__name__, filename), context)


def rst2html(
    filepath,
    template_info,
    auto_console=False,
    skip_help=False,
    skip_notes=False,
    mathjax=False,
    slide_numbers=False,
):
    # Read the infile
    with open(filepath, "rb") as infile:
        rststring = infile.read()

    presentation_dir = os.path.split(filepath)[0]

    # First convert reST to XML
    xml, dependencies = rst2xml(rststring, filepath)
    tree = etree.fromstring(xml)

    # Fix up the resulting XML so it makes sense
    sm = SlideMaker(tree, skip_notes=skip_notes)
    tree = sm.walk()

    # Pick up CSS information from the tree:
    for attrib in tree.attrib:
        if attrib.startswith("css"):

            if "-" in attrib:
                dummy, media = attrib.split("-", 1)
            else:
                media = "screen,projection"
            css_files = tree.attrib[attrib].split()
            for css_file in css_files:
                if media in ("console", "preview"):
                    # The "console" media is used to style the presenter
                    # console and does not need to be included in the header,
                    # but must be copied. So we add it as a non css file,
                    # even though it's a css-file.
                    template_info.add_resource(
                        os.path.abspath(os.path.join(presentation_dir, css_file)),
                        OTHER_RESOURCE,
                        target=css_file,
                    )
                else:
                    # Add as a css resource:
                    template_info.add_resource(
                        os.path.abspath(os.path.join(presentation_dir, css_file)),
                        CSS_RESOURCE,
                        target=css_file,
                        extra_info=media,
                    )

        elif attrib.startswith("js"):
            if attrib == "js-header":
                media = JS_POSITION_HEADER
            else:
                # Put javascript in body tag as default.
                media = JS_POSITION_BODY
            js_files = tree.attrib[attrib].split()
            for js_file in js_files:
                template_info.add_resource(
                    os.path.abspath(os.path.join(presentation_dir, js_file)),
                    JS_RESOURCE,
                    target=js_file,
                    extra_info=media,
                )

    if sm.need_mathjax and mathjax:
        if mathjax.startswith("http"):
            template_info.add_resource(
                None, JS_RESOURCE, target=mathjax, extra_info=JS_POSITION_HEADER
            )
        else:
            # Local copy
            template_info.add_resource(mathjax, DIRECTORY_RESOURCE, target="mathjax")
            template_info.add_resource(
                None,
                JS_RESOURCE,
                target="mathjax/MathJax.js?config=TeX-MML-AM_CHTML",
                extra_info=JS_POSITION_HEADER,
            )

    # Position all slides
    position_slides(tree)

    # Add the template info to the tree:
    tree.append(template_info.xml_node())

    # If the console-should open automatically, set an attribute on the document:
    if auto_console:
        tree.attrib["auto-console"] = "True"

    # If the console-should open automatically, set an attribute on the document:
    if skip_help:
        tree.attrib["skip-help"] = "True"

    # If the slide numbers should be displayed, set an attribute on the document:
    if slide_numbers:
        tree.attrib["slide-numbers"] = "True"

    # We need to set up a resolver for resources, so we can include the
    # reST.xsl file if so desired.
    parser = etree.XMLParser()
    parser.resolvers.add(ResourceResolver())

    # Transform the tree to HTML
    xsl_tree = etree.fromstring(template_info.xsl, parser)
    transformer = etree.XSLT(xsl_tree)
    tree = transformer(tree)
    result = html.tostring(tree)

    return template_info.doctype + result, dependencies


def copy_resource(filename, sourcedir, targetdir):
    if filename[0] == "/" or ":" in filename:
        # Absolute path or URI: Do nothing
        return None  # No monitoring needed
    sourcepath = os.path.join(sourcedir, filename)
    targetpath = os.path.join(targetdir, filename)

    if os.path.exists(targetpath) and os.path.getmtime(sourcepath) <= os.path.getmtime(
        targetpath
    ):
        # File has not changed since last copy, so skip.
        return sourcepath  # Monitor this file

    targetdir = os.path.split(targetpath)[0]
    if not os.path.exists(targetdir):
        os.makedirs(targetdir)

    shutil.copy2(sourcepath, targetpath)
    return sourcepath  # Monitor this file


def generate(args):
    """Generates the presentation and returns a list of files used"""

    source_files = {args.presentation}

    # Parse the template info
    template_info = Template(args.template)
    if args.css:
        presentation_dir = os.path.split(args.presentation)[0]
        target_path = os.path.relpath(args.css, presentation_dir)
        template_info.add_resource(
            args.css, CSS_RESOURCE, target=target_path, extra_info="all"
        )
        source_files.add(args.css)
    if args.js:
        presentation_dir = os.path.split(args.presentation)[0]
        target_path = os.path.relpath(args.js, presentation_dir)
        template_info.add_resource(
            args.js, JS_RESOURCE, target=target_path, extra_info=JS_POSITION_BODY
        )
        source_files.add(args.js)

    # Make the resulting HTML
    htmldata, dependencies = rst2html(
        args.presentation,
        template_info,
        args.auto_console,
        args.skip_help,
        args.skip_notes,
        args.mathjax,
        args.slide_numbers,
    )
    source_files.update(dependencies)

    # Write the HTML out
    if not os.path.exists(args.targetdir):
        os.makedirs(args.targetdir)
    with open(os.path.join(args.targetdir, "index.html"), "wb") as outfile:
        outfile.write(htmldata)

    # Copy supporting files
    source_files.update(template_info.copy_resources(args.targetdir))

    # Copy files from the source:
    sourcedir = os.path.split(os.path.abspath(args.presentation))[0]
    tree = html.fromstring(htmldata)
    for image in tree.iterdescendants("img"):
        filename = image.attrib["src"]
        source_files.add(copy_resource(filename, sourcedir, args.targetdir))
    for source in tree.iterdescendants('source'):
        filename = source.attrib['src']
        source_files.add(copy_resource(filename, sourcedir, args.targetdir))

    RE_CSS_URL = re.compile(br"""url\(['"]?(.*?)['"]?[\)\?\#]""")

    # Copy any files referenced by url() in the css-files:
    for resource in template_info.resources:
        if resource.resource_type != CSS_RESOURCE:
            continue
        # path in CSS is relative to CSS file; construct source/dest accordingly
        css_base = template_info.template_root if resource.is_in_template else sourcedir
        css_sourcedir = os.path.dirname(os.path.join(css_base, resource.filepath))
        css_targetdir = os.path.dirname(
            os.path.join(args.targetdir, resource.final_path())
        )
        uris = RE_CSS_URL.findall(template_info.read_data(resource))
        uris = [uri.decode() for uri in uris]
        if resource.is_in_template and template_info.builtin_template:
            for filename in uris:
                template_info.add_resource(
                    filename, OTHER_RESOURCE, target=css_targetdir, is_in_template=True
                )
        else:
            for filename in uris:
                source_files.add(copy_resource(filename, css_sourcedir, css_targetdir))

    # All done!

    return {os.path.abspath(f) for f in source_files if f}
